#' @title Methods for ATACProc objects
#' @description
#' You can call ATACProc objects operation methods below to
#' obtain information in objects.
#' @details
#' ATACProc is a S4 class for generating ATACProc S4 objects.
#' All ATACProc objects generated by its subclasses.
#' You can only use the ATACProc objects returned by any functions
#' rather than use ATACProc S4 class to generate object yourself.
#' @author Zheng Wei
#' @seealso

#' \code{\link{atacPipe}}
#' \code{\link{atacRemoveAdapter}}


#' @param .Object ATACProc object scalar. 
#' ATACProc object return by process functions.
#' @param atacProc Character scalar. The ATACProc subclass name 
#' for \code{printMap}.(Default: NULL). For \code{atacPrintMap}, see \code{.Object}
#' @param preProc \code{Logitcal} scalar.
#' show the available upstream processes if TRUE.(Default: FALSE)
#' @param nextProc \code{Logitcal} scalar.
#' show the available downstream processes if TRUE. (Default: TRUE)
#' @param curProc \code{Logitcal} scalar.
#' show the current process of parameter \code{.Object} if TRUE. (Default: TRUE)
#' @param display \code{Logitcal} scalar.
#' Save to pdf file if FALSE. (Default: TRUE)
#' @param item \code{Characters} scalar. 
#' The items in parameter list or report list.
#' @param ... Additional arguments, currently unused.
#' @return the function and result of functions:
#'
#' @examples
#' 
#' 
#' 
#' 
#' library(magrittr)
#' td <- tempdir()
#' options(atacConf=setConfigure("tmpdir",td))
#'
#' # Identify adapters

#' prefix<-system.file(package="esATAC", "extdata", "uzmg")

#' (reads_1 <-file.path(prefix,"m1",dir(file.path(prefix,"m1"))))
#' (reads_2 <-file.path(prefix,"m2",dir(file.path(prefix,"m2"))))
#'
#' reads_merged_1 <- file.path(td,"reads1.fastq")
#' reads_merged_2 <- file.path(td,"reads2.fastq")
#' atacproc <-
#' atacUnzipAndMerge(fastqInput1 = reads_1,fastqInput2 = reads_2) %>%
#' atacRenamer %>% atacRemoveAdapter
#'
#' subclassname<-getProcName(atacproc)
#'
#' printMap(subclassname)
#'
#' (pitems<-getParamItems(atacproc))
#' getParam(atacproc,pitems[1])
#'
#'
#' isReady(atacproc)
#' isSingleEnd(atacproc)
#' (ritems<-getReportItems(atacproc))
#' getReportVal(atacproc,ritems[1])
#'
#' clearProcCache(atacproc)
#'
#' process(atacproc)
#' 
#' @name ATACProc-class
#' @rdname ATACProc-class
#' @exportClass ATACProc
setClass(Class = "ATACProc",
         slots = list(
             fileType="list",
             paramlist = "list",
             paramlistbk = "list",
             reportVal = "list",
             procName = "character",
             completObj = "logical",
             editable = "logical",
             finish = "logical",
             singleEnd = "logical",
             logRecord= "character",
             timeStamp="character",
             timeStampPattern="character"
         ))
setMethod(f = "initialize",
          signature = "ATACProc",
          definition = function(.Object,...){
              stop("Generate ATACProc object directly is not allowed!")
              .Object
          })

setGeneric(name = "initbase",
           def = function(.Object,...){
               standardGeneric("initbase")
           })
setMethod(f = "initbase",
          signature = "ATACProc",
          definition = function(.Object,...){
              .Object@fileType <- list(fq="fq",fastq="fq",fa="fa",fasta="fa")
              .Object@paramlist <- list()
              .Object@paramlistbk <- list()
              .Object@reportVal <- list()
              .Object@procName <- ""
              .Object@completObj <- TRUE
              .Object@editable <- FALSE
              .Object@finish <- FALSE
              .Object@singleEnd <- FALSE
              .Object@logRecord <- "Generated by esATAC"
              .Object@timeStamp <- ""
              .Object@timeStampPattern <- "(\\[\\d\\d\\d\\d\\d\\d\\])*"
              .Object
          })

setGeneric(name = "init",
           def = function(.Object,procName,editable,atacProcs,...){
               standardGeneric("init")
           })
setMethod(
    f = "init",
    signature = "ATACProc",
    definition = function(.Object,procName,editable,atacProcs,...){
        .Object<-initbase(.Object)
        .Object@timeStamp <- format(Sys.time(),"[%H%M%S]")
        .Object@procName<-procName
        .Object@editable<-editable
        if(.Object@editable){
            .Object@finish <- TRUE;
        }
        argSize <- length(atacProcs)
        if(argSize>=1&&!is.null(atacProcs[[1]])){
            if(!isReady(atacProcs[[1]])){
                stop(paste(getProcName(atacProcs[[1]]),"is not ready"))
            }
            if(!checkRelation1(graphMng,getProcName(atacProcs[[1]]),procName)){
                stop(paste(getProcName(atacProcs[[1]]),"is not valid input"))
            }
            .Object@singleEnd<-isSingleEnd(atacProcs[[1]])
        }else if(argSize>=2&&!is.null(atacProcs[[2]])){
            if(!isReady(atacProcs[[2]])){
                stop(paste(getProcName(atacProcs[[2]]),"is not ready"))
            }
            if(!checkRelation2(graphMng,getProcName(atacProcs[[2]]),procName)){
                stop(paste(getProcName(atacProcs[[2]]),"is not valid input"))
            }
        }
        .Object
    }
)

setGeneric(
    name = "atacPrintMap",
    def = function(atacProc,preProc=FALSE,nextProc=TRUE,curProc=TRUE,display=TRUE){
        standardGeneric("atacPrintMap")
    }
)

#' @rdname ATACProc-class
#' @return \item{atacPrintMap}{Print map to locate the atacProc object in charflow}
#' @aliases atacPrintMap
#' @export
setMethod(
    f = "atacPrintMap",
    signature = "ATACProc",
    definition = function(atacProc,preProc=FALSE,nextProc=TRUE,curProc=TRUE,display=TRUE){
        graphPrintMap(
            graphMng,
            procName =class(atacProc),
            preProc=preProc,
            nextProc=nextProc,
            curProc=curProc,
            display = display)

    }
)

#' @rdname ATACProc-class
#' @return \item{printMap}{Print map to locate the atacProc name in charflow}
#' @export
printMap <-function(atacProc=NULL,preProc=FALSE,nextProc=TRUE,curProc=TRUE,display=TRUE){
    if(is.null(atacProc)){
        graphPrintMap(graphMng,display = display)
    }else if(is.character(atacProc)){
        graphPrintMap(
            graphMng,
            procName =atacProc,
            preProc=preProc,
            nextProc=nextProc,
            curProc=curProc,
            display = display)
    }
}



setGeneric(name = "process",
           def = function(.Object,...){
               standardGeneric("process")
           })

#' @return \item{process}{Call this function to redo processing }
#' @rdname ATACProc-class
#' @aliases process
#' @export
setMethod(f = "process",
          signature = "ATACProc",
          definition = function(.Object,...){
              msgBoxBegin()
              if(.Object@editable){
                  stop("The \"processing\" method can not be call in editable result object");
              }
              if(checkMD5Cache(.Object)){
                  message(paste0("The process:`",.Object@procName,"` was finished. Nothing to do."))
                  message("If you need to redo, please call 'clearProcCache(YourObject)'")
                  .Object@finish<-TRUE
              }else{
                  .Object <- writeLog(.Object,as.character(Sys.time()))
                  if(.Object@singleEnd){
                      .Object <- writeLog(.Object,paste0("start processing(single end data): ", .Object@procName))
                  }else{
                      .Object <- writeLog(.Object,paste0("start processing(paired end data): ", .Object@procName))
                  }
                  #.Object@paramlistbk<-.Object@paramlist
                  .Object <- processing(.Object)
                  .Object <- setFinish(.Object)
              }
              msgBoxDone()
              .Object
          })



setGeneric(name = "getProcName",
           def = function(.Object,...){
               standardGeneric("getProcName")
           })

#' @return \item{getProcName}{get ATACProc object Characher name}
#' @rdname ATACProc-class
#' @aliases getProcName
#' @export
setMethod(f = "getProcName",
          signature = "ATACProc",
          definition = function(.Object,...){
              return(.Object@procName)
          })

setGeneric(name = "getParam",
           def = function(.Object,item,...){
               standardGeneric("getParam")
           })

#' @rdname ATACProc-class
#' @return \item{getParam}{Get parameter value set by process function. 
#' See \code{getParamItems} to obtain valid item for query.}
#' @aliases  getParam
#' @export
setMethod(f = "getParam",
          signature = "ATACProc",
          definition = function(.Object,item,...){
              return(.Object@paramlist[[item]])
          })

setGeneric(name = "getParamItems",
           def = function(.Object,...){
               standardGeneric("getParamItems")
           })

#' @rdname ATACProc-class
#' @return \item{getParamItems}{Get parameter name list}
#' @aliases  getParamItems
#' @export
setMethod(f = "getParamItems",
          signature = "ATACProc",
          definition = function(.Object,...){
              return(names(.Object@paramlist))
          })

setGeneric(name = "setResultParam",
           def = function(.Object,item,val,...){
               standardGeneric("setResultParam")
           })
setMethod(f = "setResultParam",
          signature = "ATACProc",
          definition = function(.Object,item,val,...){
              if(!.Object@editable){
                  stop("This object can not be edited");
              }
              .Object@paramlist[[item]]<-val
              .Object})


setGeneric(name = "isReady",
           def = function(.Object,...){
               standardGeneric("isReady")
           })

#' @rdname ATACProc-class
#' @return \item{isReady}{Is the process ready for downstream process}
#' @aliases  isReady
#' @export
setMethod(f = "isReady",
          signature = "ATACProc",
          definition = function(.Object,...){
              if(.Object@editable){
                  return(TRUE);
              }else if(.Object@finish){
                  return(TRUE);
              }else{
                  return(FALSE);
              }
          })


setGeneric(name = "clearProcCache",
           def = function(.Object,...){
               standardGeneric("clearProcCache")
           })

#' @rdname ATACProc-class
#' @return \item{clearProcCache}{Clear cache of atacProc object}
#' @aliases  clearProcCache
#' @export
setMethod(f = "clearProcCache",
          signature = "ATACProc",
          definition = function(.Object,...){
              if(!unlink(getParamMD5Path(.Object))){
                  message("Chache has been cleared")
              }else{
                  message("Chache does not exist. Nothing has been done.")
              }
              rslist<-grep("(o|O)utput",names(.Object@paramlist))
              for(i in 1:length(rslist)){
                  unlink(.Object@paramlist[[rslist[i]]])
              }
              .Object@finish<-FALSE
              .Object
          })

setGeneric(name = "isSingleEnd",
           def = function(.Object,...){
               standardGeneric("isSingleEnd")
           })

#' @rdname ATACProc-class
#' @return \item{isSingleEnd}{Single end data if TRUE else FALSE}
#' @aliases  isSingleEnd
#' @export
setMethod(f = "isSingleEnd",
          signature = "ATACProc",
          definition = function(.Object,...){
              return(.Object@singleEnd)
          })


setGeneric(name = "getReportVal",
           def = function(.Object,item,...){
               standardGeneric("getReportVal")
           })

#' @rdname ATACProc-class
#' @return \item{getReportVal}{Get report value of item. 
#' See \code{getReportItems} to obtain valid item for query.}
#' @aliases   getReportVal
#' @export
setMethod(f = "getReportVal",
          signature = "ATACProc",
          definition = function(.Object,item,...){
              if(.Object@finish){
                  if(sum(item == getReportItemsImp(.Object))>0){
                      return(getReportValImp(.Object,item))
                  }else{
                      stop(paste0(item," is not an item of report value."))
                  }
              }else{
                  stop("Unfinished process is not available for report value.")
              }
          })


setGeneric(name = "getReportItems",
           def = function(.Object,...){
               standardGeneric("getReportItems")
           })

#' @rdname ATACProc-class
#' @return \item{getReportItems}{Get all items that can be reported}
#' @aliases getReportItems
#' @export
setMethod(f = "getReportItems",
          signature = "ATACProc",
          definition = function(.Object,...){
              if(.Object@finish){
                  return(getReportItemsImp(.Object))
              }else{
                  stop("Unfinished process is not available for report items.")
              }
          })
setGeneric(name = "getAutoPath",
           def = function(.Object,originPath,regexProcName,suffix,...){
               standardGeneric("getAutoPath")
           })
setMethod(f = "getAutoPath",
          signature = "ATACProc",
          definition = function(.Object,originPath,regexProcName,suffix,...){
              if(!is.null(originPath)){
                  prefix<-getBasenamePrefix(.Object,originPath,regexProcName)
                  return(file.path(.obtainConfigure("tmpdir"),paste0(prefix,".",getProcName(.Object),suffix)))
              }else{
                  return(NULL)
              }
          })
setGeneric(name = "paramValidation",
           def = function(.Object,...){
               standardGeneric("paramValidation")
           })
setMethod(f = "paramValidation",
          signature = "ATACProc",
          definition = function(.Object,...){
              if(!checkMD5Cache(.Object)){
                  checkAllPath(.Object)
              }
              if(!.Object@editable){
                  checkRequireParam(.Object);
              }
          })
setGeneric(name = "checkRequireParam",
           def = function(.Object,...){
               standardGeneric("checkRequireParam")
           })



setGeneric(name = "checkAllPath",
           def = function(.Object,...){
               standardGeneric("checkAllPath")
           })

setGeneric(name = "checkFileExist",
           def = function(.Object,filePath,...){
               standardGeneric("checkFileExist")
           })
setMethod(f = "checkFileExist",
          signature = "ATACProc",
          definition = function(.Object,filePath,...){
              if(!is.null(filePath)){
                  for(p in filePath){
                      if(!file.exists(p)){
                          stop(paste("error, file does not exist:",p))
                      }
                  }
              }
          })
setGeneric(name = "checkPathExist",
           def = function(.Object,filePath,...){
               standardGeneric("checkPathExist")
           })
setMethod(f = "checkPathExist",
          signature = "ATACProc",
          definition = function(.Object,filePath,...){
              if(!is.null(filePath)){
                  if(!dir.exists(dirname(filePath))){
                      stop(paste("error, path does not exist:",filePath))
                  }
              }
          })
setGeneric(name = "checkFileCreatable",
           def = function(.Object,filePath,...){
               standardGeneric("checkFileCreatable")
           })
setMethod(f = "checkFileCreatable",
          signature = "ATACProc",
          definition = function(.Object,filePath,...){
              if(!is.null(filePath)){
                  if(file.exists(filePath)){
                      .Object <- writeLog(.Object,paste("file exist:",filePath,". It may be overwrited in processing"));
                  }else if(!file.create(filePath)){
                      stop(paste("cannot create file '",filePath,"', No such file or directory or permission denied"));
                  }else{
                      unlink(filePath)
                  }
              }
          })
setGeneric(name = "checkParam",
           def = function(.Object,paramlist,paramPattern,...){
               standardGeneric("checkParam")
           })
setMethod(f = "checkParam",
          signature = "ATACProc",
          definition = function(.Object,paramlist,paramPattern,...){
              rs<-grepl(paramPattern, paramlist)
              if(sum(rs)>0){
                  banp=paste(paramlist[rs], collapse = "'/'")
                  stop(sprintf("Parameter(s) '%s' are not acceptable in paramList. it should be set as fix parameter.",banp))
              }
          })
setGeneric(name = "getSuffix",
           def = function(.Object,filePath,...){
               standardGeneric("getSuffix")
           })
setMethod(f = "getSuffix",
          signature = "ATACProc",
          definition = function(.Object,filePath,...){
              filename<-basename(filePath)
              lst=strsplit(filename,"\\.")[[1]]
              if(length(lst)==1){
                  return(NULL)
              }else{
                  return(lst[length(lst)])
              }
              return(NULL)
          })
setGeneric(name = "getSuflessFileName",
           def = function(.Object,filePath,...){
               standardGeneric("getSuflessFileName")
           })
setMethod(f = "getSuflessFileName",
          signature = "ATACProc",
          definition = function(.Object,filePath,...){
              sfx<-getSuffix(.Object,filePath=filePath)
              if(is.null(sfx)){
                  return(filePath)
              }else {
                  return(strsplit(filePath,paste0(".",sfx)))
              }
          })
setGeneric(name = "getParamMD5Path",
           def = function(.Object,...){
               standardGeneric("getParamMD5Path")
           })
setMethod(f = "getParamMD5Path",
          signature = "ATACProc",
          definition = function(.Object,...){
              paramstr=c(.Object@procName)
              for(n in sort(names(.Object@paramlist))){
                  paramstr<-c(paramstr,n)
                  paramstr<-c(paramstr,.Object@paramlist[[n]])
              }
              rslist<-grep("((o|O)utput)|((i|I)nput)",names(.Object@paramlist))
              flag = FALSE
              for(i in 1:length(rslist)){
                  filelist<-.Object@paramlist[[rslist[i]]]
                  for(j in 1:length(filelist)){
                      if(!is.character(filelist[j])){
                          flag = TRUE
                          break
                      }
                      fileinfo<-file.info(filelist[j])
                      if(is.na(fileinfo$isdir)){
                          paramstr<-c(paramstr,runif(1))
                          flag = TRUE
                          break
                      }
                      if(!fileinfo$isdir){
                          paramstr<-c(paramstr,fileinfo$size)
                      }
                  }
                  if(flag){
                      break
                  }
              }
              md5code<-substr(digest(object = paramstr,algo = "md5"),1,8)
              curtmpdir<-.obtainConfigure("tmpdir")
              md5filepath<-file.path(curtmpdir,paste(.Object@procName,md5code,"log",sep = "."))
              return(md5filepath)
          })
setGeneric(name = "setFinish",
           def = function(.Object,...){
               standardGeneric("setFinish")
           })
setMethod(f = "setFinish",
          signature = "ATACProc",
          definition = function(.Object,...){
              .Object@finish<-TRUE
              .Object<-writeLog(.Object,as.character(Sys.time()))
              .Object<-writeLog(.Object,"processing finished")
              logFilePath<-getParamMD5Path(.Object)
              write.table(.Object@logRecord,logFilePath,quote = FALSE,row.names = FALSE,col.names = FALSE)
              .Object
          })
setGeneric(name = "checkMD5Cache",
           def = function(.Object,...){
               standardGeneric("checkMD5Cache")
           })
setMethod(f = "checkMD5Cache",
          signature = "ATACProc",
          definition = function(.Object,...){
              if(file.exists(getParamMD5Path(.Object))){
                  return(TRUE)
              }else{
                  return(FALSE)
              }
          })
setGeneric(name = "getBasenamePrefix",
           def = function(.Object,filepath,words,...){
               standardGeneric("getBasenamePrefix")
           })
setMethod(f = "getBasenamePrefix",
          signature = "ATACProc",
          definition = function(.Object,filepath,words,...){
              return(basename(gsub(paste0("[.]",words,".*"),"",filepath)))
          })

setGeneric(name = "getPathPrefix",
           def = function(.Object,filepath,words,...){
               standardGeneric("getPathPrefix")
           })
setMethod(f = "getPathPrefix",
          signature = "ATACProc",
          definition = function(.Object,filepath,words,...){
              return(gsub(paste0("[.]",words,".*"),"",filepath))
          })

setGeneric(name = "writeLog",
           def = function(.Object,msg,...,isWarnning=FALSE,appendLog=TRUE,showMsg=TRUE){
               standardGeneric("writeLog")
           })
setMethod(f = "writeLog",
          signature = "ATACProc",
          definition = function(.Object,msg,...,isWarnning=FALSE,appendLog=TRUE,showMsg=TRUE){
              if(isWarnning){
                  warning(msg)
                  msg<-paste0("Warning:",msg)
              }else if(showMsg){
                  message(msg)
              }
              if(appendLog){
                  .Object@logRecord<-c(.Object@logRecord,msg)
              }else{
                  .Object@logRecord<-msg
              }
              .Object
          })
setGeneric(name = "processing",
           def = function(.Object,...){
               standardGeneric("processing")
           })

setGeneric(name = "getReportValImp",
           def = function(.Object,item,...){
               standardGeneric("getReportValImp")
           })
setMethod(f = "getReportValImp",
          signature = "ATACProc",
          definition = function(.Object,item,...){
              return(.Object@reportVal[[item]])
          })
setGeneric(name = "getReportItemsImp",
           def = function(.Object,item,...){
               standardGeneric("getReportItemsImp")
           })
setMethod(f = "getReportItemsImp",
          signature = "ATACProc",
          definition = function(.Object,item,...){
              return(names(.Object@reportVal))
          })


